<?php

namespace NoccyLabs\React\Http2\Connection;

use Evenement\EventEmitterInterface;
use Evenement\EventEmitterTrait;
use NoccyLabs\React\Http2\Frame\DataFrame;
use NoccyLabs\React\Http2\Frame\Frame;
use NoccyLabs\React\Http2\Frame\HeadersFrame;
use NoccyLabs\React\Http2\Frame\SettingsFrame;
use NoccyLabs\React\Http2\Stream\Http2Stream;
use Psr\Http\Message\ServerRequestInterface;
use React\Stream\CompositeStream;
use React\Stream\DuplexStreamInterface;
use React\Stream\ThroughStream;

/**
 * HTTP/2 Connecton Classs
 * 
 * 
 */
class Http2Connection implements EventEmitterInterface
{
    use EventEmitterTrait;

    /** @var string Emitted when a new stream is opened */
    const EVENT_OPEN = 'open';
    /** @var string Emitted when a the connection (and effectively all its streams) are closed */
    const EVENT_CLOSE = 'close';

    const EVENT_ERROR = 'error';

    /** @var array<int,Http2Stream> The active streams, even for s2c, odd for c2s */
    private array $streams = [];

    /** @var int The last used stream identifier */
    private int $lastStream = 0;

    /** @var int Max number of streams that can be opened; assigned from settings frame */
    private int $maxStream = 0;

    private ServerRequestInterface $request;

    /** @var DuplexStreamInterface The connection to the client */
    private DuplexStreamInterface $connection;

    private DuplexStreamInterface $responseStream;

    private ?SettingsFrame $settings = null;

    private string $buffer = '';

    public function __construct(ServerRequestInterface $request, ?SettingsFrame $http2Settings)
    {
        $this->request = $request;
        $this->settings = $http2Settings;

        $responseInputStream = new ThroughStream();
        $responseOutputStream = new ThroughStream();

        $this->responseStream = new CompositeStream($responseOutputStream, $responseInputStream);
        $this->connection = new CompositeStream($responseInputStream, $responseOutputStream);

        $this->connection->on('data', function ($data) {
            $this->buffer .= $data;
            $frame = Frame::parseFrame($this->buffer);
            $this->handleHttp2Frame($frame);
        });
       
    }

    private function handleHttp2Frame(Frame $frame)
    {
        switch ($frame->getFrameType()) {
            case Frame::FRAME_HEADERS:
                $this->handleHttp2HeadersFrame($frame);
                break;
            case Frame::FRAME_DATA:
                $this->handleHttp2DataFrame($frame);
                break;
            case Frame::FRAME_PING:
                $this->handleHttp2PingFrame($frame);
                break;
        }
    }

    private function handleHttp2HeadersFrame(HeadersFrame $frame)
    {
        $method = $frame->headers->getHeaderLine(':method');
        $scheme = $frame->headers->getHeaderLine(':scheme');
        $path = $frame->headers->getHeaderLine(':path');
    }

    private function handleHttp2DataFrame(DataFrame $frame)
    {

    }

    private function handleHttp2PingFrame(DataFrame $frame)
    {

    }

    /**
     * Create a pair of streams, one for incoming and one for outgoing data.
     *
     * @return Http2Stream[]
     */
    private function createStreamPair(): array
    {

    }

    /**
     * When a new stream is opened, clean up older streams that are idle or closed.
     * 
     * @return void
     */
    private function doStreamHouseKeeping(): void
    {

    }

    public function getResponseStream(): DuplexStreamInterface
    {
        return $this->responseStream;
    }
}